/*
 * Copyright (c) 2009,2013 CNRS/LAAS
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */
#ifndef _IVMOBSTRUCT_H
#define _IVMOBSTRUCT_H

/* Main modes of the module */
typedef enum IVMOB_MODE {
	IVMOB_MOTORS_OFF,	/* reset, inactive */
	IVMOB_IDLE,		/* active but no command */
	IVMOB_TRACK,		/* moving */
	IVMOB_MANUAL,		/* active but remotely contolled */
	IVMOB_EMERGENCY		/* emergency mode */
} IVMOB_MODE;


typedef struct IVMOB_STATUS_STR {
	IVMOB_MODE rs_mode;
	int operational_mode;	/* from robot */
	double battery_level;
	double temperature;
	double v, w;
	double front_left_wheel_speed;
	double front_right_wheel_speed;
	double rear_left_wheel_speed;
	double rear_right_wheel_speed;
	double front_left_current;
	double front_right_current;
	double rear_left_current;
	double rear_right_current;
	int mmi_state;
	int safety_code;
} IVMOB_STATUS_STR;

/* Kinematics of the robot */
typedef struct IVMOB_KINEMATICS_STR {
	double wheelRadius;	
	double axisWidth;
} IVMOB_KINEMATICS_STR;

/* Current dynamic parameters, from the nvram */
typedef struct IVMOB_DYNAMIC_STR {
	double vMax;
	double wMax;
	double linAccelMax;
	double angAccelMax;
} IVMOB_DYNAMIC_STR;

/* Coefficient proportionnel a la vitesse pour calculer l'erreur sur 
   le deplacement des roues */
typedef struct IVMOB_VAR_PARAMS {
	double coeffLin;
	double coeffAng;
	double coeffLinAng;
} IVMOB_VAR_PARAMS;

/* Track parameters */
typedef struct IVMOB_TRACK_PARAMS {
	GENPOS_POSTER_NAME posterName;
	GENPOS_TRACK_MODE mode;
} IVMOB_TRACK_PARAMS;

/*----------------------------------------------------------------------*/
/* 3D */

/* a GENPOS_CART_STATE-like struct in 3D */
/* Same thing with 4 more elements: z,zC,pitch,roll */
typedef struct IVMOB_CART_3DSTATE {
	double xRef;                  /* position  */
	double yRef;                  /* control point */
	double zRef;
	double xRob;                 /* position */
	double yRob;                 /* robot center */
	double zRob;
	double theta;              /* robot orientation*/
	double pitch;              /* robot's pitch (site) */
	double roll;               /* robot's roll (gite) */
	double v;                  /* linear velocity */
	double vt;
	double w;                  /* angular velocity */
} IVMOB_CART_3DSTATE;

typedef struct IVMOB_CART_3DCONFIG {
	double x;
	double y;
	double z;
	double theta;
} IVMOB_CART_3DCONFIG;

/*----------------------------------------------------------------------*/
/* Odometry Method */

typedef enum IVMOB_ODOMETRY_MODE {
	IVMOB_ODO_2D, /* Standard 2D odometry */
	IVMOB_ODO_3D  /* 3D odometry using Attitude Estimation 
			  provided by IMU */
} IVMOB_ODOMETRY_MODE;

#define	IVMOB_MAX_STRLEN	64
#define IVMOB_DEFAULT_ATTITUDE_POSTER "imuAttitudeEstimation"

typedef struct IVMOB_ODOMETRY_METHOD {
	IVMOB_ODOMETRY_MODE mode;
	int unused;
	char attitudePosterName[IVMOB_MAX_STRLEN];
} IVMOB_ODOMETRY_METHOD;

/*----------------------------------------------------------------------*/
/* Gyro */

#ifdef GYRO
#include <gyroLib/gyroTypes.h>
#endif

typedef enum IVMOB_GYRO_MODE {
	IVMOB_GYRO_OFF,
	IVMOB_GYRO_ON_IF_MOTION,
	IVMOB_GYRO_ON
} IVMOB_GYRO_MODE;

typedef struct IVMOB_GYRO_PARAMS {
	IVMOB_GYRO_MODE mode;
	char port[32];
#ifdef GYRO
	GYRO_TYPE type;
#endif
	double latitude;
	double woffset;
} IVMOB_GYRO_PARAMS;

typedef struct IVMOB_GYRO {
	IVMOB_GYRO_PARAMS params;
	int gyroOn;
	IVMOB_GYRO_MODE currentMode;
	double gyroToRobotOffset;
	double gyroTheta;
	double robotTheta;
	double gyroOmega;
} IVMOB_GYRO;


typedef struct IVMOB_GYRO_ASSERV {
	int enabled; // is asserv enabled
	
	int first; // first call to this function ?
	int straight; // is in straight mode ?
	double straight_angle; // angle for straight mode
	double prev_reference;
	double prev_measure;
	double prev_command;
	double prev_t;
	double integral_s;
	double jump_t; // last jump time
} IVMOB_GYRO_ASSERV;

typedef struct IVMOB_MAX_ACCEL {
	double prev_vel_command;
	double prev_vel_command_t;
} IVMOB_MAX_ACCEL;

#endif
